<!DOCTYPE html>
<html lang="en">
<head>
    <title>Survey </title>
    <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.5.2/css/bootstrap.min.css" integrity="sha384-JcKb8q3iqJ61gNV9KGb8thSsNjpSL0n8PARn9HuZOnIxN0hoP+VmmDGMN5t9UJ0Z" crossorigin="anonymous">
    <style>{{ theme_css|safe }}</style>
</head>
<body>
   <div class="survey container-fluid">
   {% block survey %}{% endblock %}
   </div>
</body>
</html>